#!/bin/sh

#set -x

function dec_variable {
# declaration of varibles 
yum_repo_path="/u001/mrepo/CentOS_6-x86_64/RPMS.fiberlink/"
filename="rpm_list.txt"
patch_applied_logdir="/u001/Patch_Applied/2.1.0.4/ospatch/log" 
mkdir -p $patch_applied_logdir
patch_applied_backupdir="/u001/Patch_Applied/2.1.0.4/ospatch/backup"
mkdir -p $patch_applied_backupdir
chown -R automation_prod:automation_prod "/u001/Patch_Applied/2.1.0.4/"
chown -R automation_prod:automation_prod "/u001/patches/maas360patch/2.1.0.4"

puppet_path="/u001/puppet/1.10.40.0.onprem/maas/machines/files"
puppet_sudoers="sudoers"

LOG="$patch_applied_logdir/apply_patch.log"
failed=0

}

#copy rpms to repo dir
function copy_rpm {
    echo "Copying rpms to $yum_repo_path" >> ${LOG} 2>&1
    /bin/cp -f rpms/*.rpm $yum_repo_path
	if [ `echo $?` -ne 0 ];then      
 
	echo "Error ::Failed to copy packages to $yum_repo_path . " 
       echo "Error ::Failed to copy packages to $yum_repo_path . " >> ${LOG} 2>&1
       exit 1
     fi

}


function logrotate {

	date=`date +%e%m%y%H%M%S`

	[ -f ${LOG} ] && mv ${LOG} ${LOG}_$date > /dev/null 2>&1

	echo -e "\n\n" >> ${LOG}
	echo "Script Execution started"	
	echo "Script Execution started" >> ${LOG} 2>&1

}


function backup_sudoers {
[[ ! -f $patch_applied_backupdir/$puppet_sudoers-bkp ]] && { 

echo "Taking backup of $patch_applied_backupdir/$puppet_sudoers file" >> ${LOG} 2>&1
/bin/cp -pf $puppet_path/$puppet_sudoers $patch_applied_backupdir/$puppet_sudoers-bkp >> ${LOG} 2>&1 
   if [ `echo $?` -ne 0 ];then 
    echo " Warning::Failed to copy  $puppet_path/$puppet_sudoers file"
    echo " Warning::Failed to copy  $puppet_path/$puppet_sudoers file" >> ${LOG} 2>&1
   fi    

}

}

function update_sudoers {

grep -w "appuser ALL=NOPASSWD: /usr/bin/yum -y update" "${puppet_path}/${puppet_sudoers}" >> ${LOG} 2>&1
if [ `echo $?` -ne 0 ]; then

cat >> "${puppet_path}/${puppet_sudoers}" <<EOF
appuser ALL=NOPASSWD: /usr/bin/yum -y update
EOF

echo "Executing Puppet" >> ${LOG} 2>&1
echo "Executing Puppet"

for host in op1infra1-0 op1portalapp1-0 op1standalone1-0 op1svcapp1-0

do
su - automation_prod -c "fab -H $host -u appuser -- sudo  /usr/bin/puppet agent -t" >> ${LOG} 2>&1

done

fi


}

function Install_createrepo {
# install pre-requisites packages 

pushd ${yum_repo_path} >> ${LOG} 2>&1
echo "Installing Createrepo"
echo "Installing Createrepo" >> ${LOG} 2>&1
rpm -ivh python-deltarpm-* deltarpm-* createrepo-* >> ${LOG} 2>&1
popd >> ${LOG} 2>&1

}

function Run_createrepo {

echo "Running Createrepo..."
echo "Running Createrepo" >> ${LOG} 2>&1
createrepo "${yum_repo_path}"  >> ${LOG} 2>&1
 if [ `echo $?` -ne 0 ];then
    echo "Failed to create repo at $yum_repo_path ."
    echo "Failed to create repo at $yum_repo_path ." >> ${LOG} 2>&1
    exit 1
fi

}

function rpm_install {


#yum update on infra
#echo "Executing /usr/bin/yum clean all" >> ${LOG} 2>&1
#/usr/bin/yum clean all >> ${LOG} 2>&1
#echo "Executing /usr/bin/yum -y update"  >> ${LOG} 2>&1
#/usr/bin/yum -y update >> ${LOG} 2>&1 
#if [ `echo $?` -ne 0 ];then
 #   echo "Failed to update rpms on infra." 
  #  echo "Failed to update rpms on infra."  >> ${LOG} 2>&1
   # exit 1
#fi


#yum update for 3 boxes 

echo "Updating RPMs.."

echo "Executing /usr/bin/yum clean all on op1infra1-0,op1portalapp1-0,op1standalone1-0,op1svcapp1-0" >> ${LOG} 2>&1
su - automation_prod -c "fab -H op1infra1-0,op1portalapp1-0,op1standalone1-0,op1svcapp1-0 -u appuser -- sudo /usr/bin/yum clean all" >> ${LOG} 2>&1 
echo "Executing /usr/bin/yum -y update on op1infra1-0,op1portalapp1-0,op1standalone1-0,op1svcapp1-0" >> ${LOG} 2>&1
su - automation_prod -c "fab -H op1infra1-0,op1portalapp1-0,op1standalone1-0,op1svcapp1-0 -u appuser -- sudo /usr/bin/yum -y update" >> ${LOG} 2>&1 
if [ `echo $?` -ne 0 ];then    
echo "RPM update on target machines failed. Restore all 4 VMs from VM backup"
echo "RPM update on target machines failed. Restore all 4 VMs from VM backup"  >> ${LOG} 2>&1
    exit 1 
fi


}
#verify that the packages are updated .

function rpm_verification {


echo "Verifying RPMs..." >> ${LOG} 2>&1

su - automation_prod -c "fab -H op1infra1-0 -u appuser -- /bin/rpm -qa" > $patch_applied_logdir/installed_packages_op1infra1-0 2>&1
if [ `echo $?` -ne 0 ];then
    echo "Executing fab command on op1infra1-0 failed ." >> ${LOG} 2>&1
    exit 1
fi

cat $patch_applied_logdir/installed_packages_op1infra1-0 >> ${LOG} 2>&1

su - automation_prod -c "fab -H op1portalapp1-0 -u appuser -- /bin/rpm -qa" > $patch_applied_logdir/installed_packages_op1portalapp1-0 2>&1
if [ `echo $?` -ne 0 ];then
    echo "Executing fab command on op1portalapp1-0 failed ." >> ${LOG} 2>&1
    exit 1
fi


cat $patch_applied_logdir/installed_packages_op1portalapp1-0 >> ${LOG} 2>&1

su - automation_prod -c "fab -H op1standalone1-0 -u appuser -- /bin/rpm -qa" > $patch_applied_logdir/installed_packages_op1standalone1-0 2>&1
if [ `echo $?` -ne 0 ];then
    echo "Executing fab command on op1standalone1-0 failed ." >> ${LOG} 2>&1
    exit 1
fi

cat $patch_applied_logdir/installed_packages_op1standalone1-0  >> ${LOG} 2>&1

su - automation_prod -c "fab -H op1svcapp1-0 -u appuser -- /bin/rpm -qa" > $patch_applied_logdir/installed_packages_op1svcapp1-0 2>&1
if [ `echo $?` -ne 0 ];then
    echo "Executing fab command on op1svcapp1-0 failed ." >> ${LOG} 2>&1
    exit 1
fi

cat $patch_applied_logdir/installed_packages_op1svcapp1-0 >> ${LOG} 2>&1


for machine in op1infra1-0 op1portalapp1-0 op1standalone1-0 op1svcapp1-0
do
        for rpm_name in `cat rpm_list.txt`
        do
	
	
        grep -w "$rpm_name" $patch_applied_logdir/installed_packages_"$machine" >> /dev/null 2>&1
        if [ `echo $?` -ne 0 ];then
		   echo " $rpm_name on $machine failed to update. "
                   echo " $rpm_name on $machine failed to update" >> ${LOG} 2>&1 
                failed=1
        fi
        done
	
	for rpm_name in `grep $machine misc.txt | awk '{for (i=2; i<NF; i++) printf $i " "; print $NF}'` 
	do
	#echo $rpm_name
	grep -w "$rpm_name" $patch_applied_logdir/installed_packages_"$machine" >> /dev/null 2>&1
        if [ `echo $?` -ne 0 ];then
                   echo " $rpm_name on $machine failed to update"
                   echo " $rpm_name on $machine failed to update" >> ${LOG} 2>&1 
                failed=1
        fi
        done

	
done


if [ $failed -eq 1 ];
then
        echo "RPM update on one or more VMs failed. Please refer ${LOG} for more details. Restore all 4 VMs from VM backup"
        exit 1
fi


}

# main starts here

## Check if id is root
if (( $EUID != 0 )); then
    echo "Please run as root"
    exit
fi

dec_variable
logrotate
copy_rpm
backup_sudoers
update_sudoers
Install_createrepo
Run_createrepo
rpm_install
rpm_verification
echo "Patch Applied Succesfully"
echo "Patch Applied Succesfully" >> ${LOG} 2>&1
echo "Status=0"
exit 0
